<?php

namespace Drupal\simple_menu_permissions\Access;

use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResultAllowed;
use Symfony\Component\Routing\Route;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Access checker for menu and menu item routes.
 */
class MenuRoutesAccessChecker implements AccessInterface {

  /**
   * Custom access checker.
   */
  public function checkAccess(Route $route, RouteMatchInterface $route_match, AccountInterface $account): AccessResultNeutral|\Drupal\Core\Access\AccessResult|AccessResultAllowed {
    // Get the menu entity from the route match.
    $menu = $route_match->getParameter('menu');

    return match($route_match->getRouteName()) {
      'entity.menu.edit_form' => AccessResultAllowed::allowedIf($account->hasPermission('edit ' . $menu->id() . ' menu')),
      'entity.menu.delete_form' => AccessResultAllowed::allowedIf($account->hasPermission('delete ' . $menu->id() . ' menu')),
      'entity.menu.add_link_form' => AccessResultAllowed::allowedIf($account->hasPermission('add new links to ' . $menu->id() . ' menu')),
      default => AccessResultAllowed::neutral(),
    };
  }

}
