<?php

namespace Drupal\simple_menu_permissions\Form;

use Drupal\menu_ui\MenuForm as CoreMenuForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

/**
 * Extends the base form for menu add and edit forms.
 *
 * @internal
 */
class MenuForm extends CoreMenuForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    $canAddNewLinks = $this->currentUser()->hasPermission('add new links to ' . $this->entity->id() . ' menu');
    $canEditLinks = $this->currentUser()->hasPermission('edit links in ' . $this->entity->id() . ' menu');

    if (isset($form['links']['links'])) {
      if (!$canAddNewLinks) {
        // Remove the "Add link" button if the user does not have the permission to add new links.
        if (isset($form['links']['links']['#empty'])) {
          $form['links']['links']['#empty'] = t('There are no menu links yet.');
        }
      }

      // Disable the table drag if the user does not have the permission to edit links.
      if (!$canEditLinks) {
        $form['links']['links']['#tabledrag'] = NULL;
      }

      $menuItemKeys = Element::children($form['links']['links']);

      foreach ($menuItemKeys as $key) {
        // Remove the "Add child" button if the user does not have the permission to add new links.
        if (isset($form['links']['links'][$key]['operations']['#links']['add-child'])) {
          unset($form['links']['links'][$key]['operations']['#links']['add-child']);
        }

        // Disable the "Enabled" checkbox if the user does not have the permission to edit links.
        if (isset($form['links']['links'][$key]['enabled'])) {
          $form['links']['links'][$key]['enabled']['#disabled'] = !$canEditLinks;
        }

        // Disable the "Weight" field if the user does not have the permission to edit links.
        if (isset($form['links']['links'][$key]['weight'])) {
          $form['links']['links'][$key]['weight']['#disabled'] = !$canEditLinks;
        }

        // @todo Remove this once Drupal 9 support is dropped and test if it still works in Drupal 10.
        // Remove the "Edit" button if the user does not have the permission to edit links.
        if (!$this->currentUser()->hasPermission('edit links in ' . $this->entity->id() . ' menu')) {
          if (isset($form['links']['links'][$key]['operations']['#links']['edit'])) {
            unset($form['links']['links'][$key]['operations']['#links']['edit']);
          }
        }

        // @todo Remove this once Drupal 9 support is dropped and test if it still works in Drupal 10.
        // Remove the "Delete" button if the user does not have the permission to delete links.
        if (!$this->currentUser()->hasPermission('delete links in ' . $this->entity->id() . ' menu')) {
          if (isset($form['links']['links'][$key]['operations']['#links']['delete'])) {
            unset($form['links']['links'][$key]['operations']['#links']['delete']);
          }
        }
      }
    }

    return $form;
  }

}
