<?php

namespace Drupal\simple_menu_permissions;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\menu_ui\MenuListBuilder as CoreMenuListBuilder;

/**
 * Extends the core class to build a listing of menu entities.
 *
 * @see \Drupal\system\Entity\Menu
 * @see menu_entity_info()
 */
class MenuListBuilder extends CoreMenuListBuilder {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $account;

  /**
   * Constructs a new MenuListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
    parent::__construct($entity_type, $storage);
    $this->account = \Drupal::currentUser()->getAccount();
  }

  /**
   * {@inheritdoc}
   */
  public function load(): array {
    // Load the menu entities.
    $entities =  parent::load();

    // Filter out the menus that the user does not have permission to view.
    return array_filter($entities, function ($menu) {
      return $this->account->hasPermission('view ' . $menu->id() . ' menu in menu list');
    });
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    // Get the default operations.
    $operations = parent::getDefaultOperations($entity);

    // Remove the 'add' operation if the user does not have permission to add new links to the menu.
    if (isset($operations['add']) && !$this->account->hasPermission('add new links to ' . $entity->id() . ' menu')) {
      unset($operations['add']);
    }

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function render(): array {
    $build = parent::render();

    if (isset($build['table']['#empty'])) {
      $build['table']['#empty'] = $this->t('There are no @label yet, or you do not have the permission to view a menu.', ['@label' => t('menus')]);
    }

    return $build;
  }

}
