<?php

namespace Drupal\simple_menu_permissions;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuParentFormSelector;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\StringTranslation\TranslationInterface;

/**
 * Decorator of the menu parent form selector service.
 */
class MenuParentFormSelectorDecorator extends MenuParentFormSelector {

  /**
   * The simple menu permissions controller.
   *
   * @var \Drupal\Core\Controller\ControllerBase
   */
  protected ControllerBase $simpleMenuPermissionsController;

  /**
   * {@inheritdoc}
   */
  public function __construct(ControllerBase $simple_menu_permissions_controller, MenuLinkTreeInterface $menu_link_tree, EntityTypeManagerInterface $entity_type_manager, TranslationInterface $string_translation) {
    parent::__construct($menu_link_tree, $entity_type_manager, $string_translation);
    $this->simpleMenuPermissionsController = $simple_menu_permissions_controller;
  }

  /**
   * {@inheritdoc}
   */
  public function getParentSelectOptions($id = '', ?array $menus = NULL, ?CacheableMetadata &$cacheability = NULL): array {
    if (!isset($menus)) {
      $menus = $this->getMenuOptions();
    }

    $menus = $this->filterMenus($menus);

    return parent::getParentSelectOptions($id, $menus, $cacheability);
  }

  /**
   * {@inheritdoc}
   */
  public function parentSelectElement($menu_parent, $id = '', ?array $menus = NULL): array {
    if (!isset($menus)) {
      $menus = $this->getMenuOptions();
    }

    $menus = $this->filterMenus($menus);

    $element = parent::parentSelectElement($menu_parent, $id, $menus);

    if ($id !== '' && !isset($element['#default_value']) && $menu_parent) {
      $element['#disabled'] = TRUE;
      $element['#readonly'] = TRUE;
      $element['#restricted'] = TRUE;
      $element['#options'][$menu_parent] = '- Restricted -';
      $element['#default_value'] = $menu_parent;
    }

    return $element;
  }

  /**
   * Filter the menus based on the user's permissions.
   */
  private function filterMenus(array $menus): array {
    $allowedMenus = $this->simpleMenuPermissionsController->getSelectableMenus();
    return array_intersect_key($menus, $allowedMenus);
  }

}
